#!/usr/bin/python -u
#
'''
The client part of remote Python interpreter.
'''
import hashlib
import os
import re
import socket
import sys

try:
    bytearray
except NameError:
    # for older Python, requires compatibility.py from here
    from .compatibility import bytearray


# check if password prompt echo can be suppressed
suppress = ''
if sys.platform == 'linux2':
    if hasattr(sys.stdout, 'fileno'):
        if os.isatty(sys.stdout.fileno()):
            suppress = '\033]8m'


def interact(family, address):
    '''
    Start interactive remote interpreter residing in specified network
    address belonging to given address family. (see socket)
    '''
    connection = socket.socket(family, socket.SOCK_STREAM)
    connection.connect(address)
    recvbuffer = bytearray()
    while '\n' not in recvbuffer:
        newdata = connection.recv(4096)
        recvbuffer.extend(newdata)
    salt = str(recvbuffer[:-1])
    recvbuffer[:] = []
    sys.stdout.write('Password: ' + suppress)
    password = raw_input()
    sys.stdout.write('\r')
    pwhash = hashlib.sha1(password).hexdigest()
    trial = hashlib.sha1(pwhash + salt).hexdigest()
    connection.sendall(trial+'\n')
    # assume connected, server will message if not :p
    try:
        while True:
            while '\n' not in recvbuffer:
                newdata = connection.recv(4096)
                if not newdata: return
                recvbuffer.extend(newdata)
            while '\n' in recvbuffer:
                boundary = recvbuffer.find('\n')
                line = str(recvbuffer[:boundary])
                recvbuffer[:boundary+1] = []
                new_input = handle_line(line)
                if new_input is not None:
                    connection.sendall(new_input+'\n')
    except EOFError:
        print
        connection.close()


def handle_line(line):
    '''
    Handle stuff from server.
    '''
    if len(line) < 2 or line[1] != ':':
        # malformed input line
        sys.stderr.write('Got erroneous line %r\n' % line)
    msgtype, msg = line.split(':', 1)
    # print msgtype, repr(msg)
    if msgtype == '1':
        # normal prompt
        return raw_input('>>> ')
    elif msgtype == '2':
        # continue prompt
        return raw_input('... ')
    elif msgtype == 'O':
        # output
        sys.stdout.write(msg+'\n')
    elif msgtype == 'E':
        # output on stderr
        sys.stderr.write(msg+'\n')
    elif msgtype == 'N':
        # notification
        sys.stderr.write(msg+'\n')
    elif msgtype == 'C':
        # connection close
        raise RuntimeError('Connection closed: '+msg)
    else:
        # WTF?
        raise RuntimeError('WTF? ' + line)


def main(args):
    '''
    Someone was lazy to fire up Python console to import the module and
    call interact() but instead just ran this module as a script...
    Well, try to figure out what the user means.
    '''
    if len(args) != 2:
        sys.stderr.write('Usage: %s address\n' % args[0])
        return 1
    raw_address = args[1]
    # try to figure out the address type
    if re.match(r'^[^/\\:]+:\d{1,5}$', raw_address):
        # IPv4 most likely, but with hostname could be IPv6 too...
        # hmm FIXME when you need it fixed
        family = socket.AF_INET
        host, port = raw_address.rsplit(':', 1)
        address = host, int(port)
    elif re.match(r'^\[[\d:]+\]:\d{1,5}$', raw_address):
        # IPv6 looks like
        family = socket.AF_INET6
        host, port = raw_address.rsplit(':', 1)
        address = host[1:-1], int(port)
    else:
        # for now assume AF_UNIX
        family = socket.AF_UNIX
        address = raw_address
    interact(family, address)
    return 0

if __name__ == '__main__':
    # executed as a script
    sys.exit(main(sys.argv))

